#!/bin/bash

# SUpload (Screenshot-Upload)
version="1.1"

## Colours
r='\033[0;31m'
nc='\033[0m'

## Default  Configs
## WARNING: Do not change these files here, change them in the config file

sfw=false
debug=false
sshot="maim"
file_format="date +%F-%T.png"
file_dir="${HOME}/Pictures/screenshots"
browser=false
clipboard=true
notification=true
log=true
log_file="${HOME}/.supload.log"
curl_options="--progress-bar"

## Read from user config if exists, else read from system-wide (just use inbuilt defaults if none)
### User config takes priority over system-wide 
config=$HOME/.config/supload/settings.conf 
if [ -f "${config}" ] ; then
  source "${config}"
elif [ -f "/etc/supload.conf" ] ; then
  source "/etc/supload.conf"
fi

## Check dependencies (in case of a bad installation)
type jq >/dev/null 2>&1 || { echo -e >&2 "${r}ERROR:${nc} I require jq but it's not installed.  Aborting."; exit 1; }
type curl >/dev/null 2>&1 || { echo -e >&2 "${r}ERROR:${nc} I require curl but it's not installed.  Aborting."; exit 1; }

usage() {
cat <<EOF
Usage: supload [OPTIONS] [FILE]

Options:

  -f      Safe for work mode, (uses volkor.me)
  -h      Show this menu
  -o      Automatically open in browser
  -s      Screenshot mode (maim Selection)
  -V      Verbose
  -v      Shows current version of SUpload

Examples: https://git.volkor.me/Volkor/SUpload

EOF
exit 0
}

## Actual file uploading
upload_file() {
  if [[ "$sfw" = "true" ]] ; then
    base_url="https://volkor.me/"
  else
    base_url="https://thecum.zone/"
  fi
  $debug && echo -e "${r}DEBUG:${nc} BASE_URL: $base_url"

  ## Upload to server
  echo "Uploading File: ($1)"
	response=$(curl $curl_options -F file=@"$1" "$base_url") || { echo -e >&2 "${r}ERROR:${nc} Curl upload failure. (Invalid file, or bad internet connection)"; exit 1; }
  $debug && echo -e "${r}DEBUG:${nc} RESPONSE: $response"
  
  url=$(echo "$response" | jq -r '.["url"]')
  deletionlink=$(echo "$response" | jq -r '.["deletionurl"]')
  $debug && echo -e "${r}DEBUG:${nc} URL: $url"

  ## Browser, clipboard and logging.
  if [ "$browser" = "true" ] ; then
    echo "Opening browser"
    hash xdg-open && xdg-open "$url" &>/dev/null || echo "${r}ERROR:${nc} xdg-open was not found, or failed to run."
  fi

  if [ "$clipboard" = "true" ] && [ -x "$(command -v xclip)" ]; then
    echo "${url}" | tr -d '\n' | xclip -selection clipboard ; echo "Copied URL to clipboard";
  fi
$debug && echo -e "${r}DEBUG:${nc} BROWSER: $browser"
$debug && echo -e "${r}DEBUG:${nc} CLIPBOARD: $clipboard"

  if [ -f "${log_file}" ]; then
    if [ "$log" = "true" ]; then
      echo -n "$(date +"[%Y-%m-%d %H:%M:%S]")" >> "$log_file" && echo " Upload Response: $response" >> "$log_file"
    fi
  else
    touch "$log_file"
  fi
  $debug && echo -e "${r}DEBUG:${nc} LOG: $log"
  $debug && echo -e "${r}DEBUG:${nc} LOG_FILE: $log_file"
  upload_finished
}

upload_finished() {
  echo "URL: ${url}"
  echo "Deletion URL: ${deletionlink}"
  if [ "$notification" = "true" ] ; then
    notify-send "File uploaded." "URL: ${url}" & exit 0
  fi
  exit 0
}

while getopts "fhosvV" opt; do
  case $opt in
    f)
	    sfw=true
      ;;
    h)
      usage
      ;;
    o)
	    browser=true
      ;;
    s)
      screenshot=true
      ;;
    V)
      debug=true
      ;;
    v)
      echo "SUpload Version: "$version
      exit 0
      ;;
    *)
      echo -e "${r}ERROR:${nc} Invalid Option."
  esac
done

file="${*:$OPTIND}"
$debug && echo -e "${r}DEBUG:${nc} OPTIONS: $*"

if [ "$screenshot" = "true" ] ; then
  echo "Click and hold to select screenshot area or click for window."
  screen_path="$file_dir/$($file_format)"
  $debug && echo -e "${r}DEBUG:${nc} SCREEN_PATH: $screen_path"

  case $sshot in
    maim)
      screen_prog="maim -s -o" ;;
    scrot)
      screen_prog="scrot -s" ;;
    *)
      echo -e "${r}ERROR:${nc} Invalid SSHOT in config, must be scrot or maim."
      exit 1
      ;;
  esac
  $debug && echo -e "${r}DEBUG:${nc} SSHOT: $sshot"

  if ! $screen_prog "${screen_path}"; then
    echo -e "${r}ERROR:${nc} Failed to take screenshot (maim crashed/user exited)"
    exit 1
  fi
  upload_file "${screen_path}"
fi

if [ "$1" = "" ] ; then
  echo -e "${r}ERROR:${nc} You must supply either STDIN (-) or a file!"
  exit 1
fi

upload_file "$file"